/*
 * inetd/daemonize.c --
 *
 *      This module implements daemonization of a process, either by
 *      calling a suitable library function or providing the
 *      functionality inline.
 */

#define _POSIX_C_SOURCE 2
#define _DEFAULT_SOURCE
#define _DARWIN_C_SOURCE

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <syslog.h>

#include "inetd.h"

void
inetd_daemonize()
{
#if defined(__linux__)
    if (daemon(0,0)) {
	syslog(LOG_ERR, "daemon() failed - exiting");
	exit(EXIT_FAILURE);
    }
#else
    pid_t pid;

    if (chdir("/")) {
	syslog(LOG_ERR, "cannot change dir to /: %m - exiting");
	exit(EXIT_FAILURE);
    }
    
    if (! freopen("/dev/null","w",stdout)) {
	syslog(LOG_ERR, "cannot redirect stdout to /dev/null: %m - exiting");
	exit(EXIT_FAILURE);
    }
    
    if (! freopen("/dev/null","w",stderr)) { 
	syslog(LOG_ERR, "cannot redirect stderr to /dev/null: %m - exiting");
	exit(EXIT_FAILURE);
    }
    if (! freopen("/dev/null","r",stdin)) {
	syslog(LOG_ERR, "cannot redirect stdin to /dev/null: %m - exiting");
	exit(EXIT_FAILURE);
    }
    
    /* fork and parent exits immediately causing init to adopt child */

    pid = fork();
    if (pid == -1) {
	syslog(LOG_ERR, "fork() failed: %m - exiting");
	exit(EXIT_FAILURE);
    }
    if (pid) {
	exit(EXIT_SUCCESS);
    }
#endif
}

